@@ -70,6 +70,20 @@ class RelativeTime(object):
   def __init__(self, step=1):
     self._step = step
 
+  def step_size(self, t0=None, t1=None):
+    '''
+    Return the time in seconds of a step. If a begin and end timestamp,
+    return the time in seconds between them after adjusting for what buckets
+    they alias to. If t1 and t0 resolve to the same bucket,
+    '''
+    if t0!=None and t1!=None:
+      tb0 = self.to_bucket( t0 )
+      tb1 = self.to_bucket( t1, steps=1 ) # NOTE: "end" of second bucket
+      if tb0==tb1:
+        return self._step
+      return self.from_bucket( tb1 ) - self.from_bucket( tb0 )
+    return self._step
+
   def to_bucket(self, timestamp, steps=0):
     '''
     Calculate the bucket from a timestamp, optionally including a step offset.
@@ -138,6 +152,22 @@ class GregorianTime(object):
   def __init__(self, step='daily'):
     self._step = step
 
+  def step_size(self, t0, t1=None):
+    '''
+    Return the time in seconds for each step. Requires that we know a time
+    relative to which we should calculate to account for variable length
+    intervals (e.g. February)
+    '''
+    tb0 = self.to_bucket( t0 )
+    if t1:
+      tb1 = self.to_bucket( t1, steps=1 ) # NOTE: "end" of second bucket
+    else:
+      tb1 = self.to_bucket( t0, steps=1 )
+
+    # Calculate the difference in days, then multiply by simple scalar
+    days = (self.from_bucket(tb1, native=True) - self.from_bucket(tb0, native=True)).days
+    return days * SIMPLE_TIMES['d']
+
   def to_bucket(self, timestamp, steps=0):
     '''
     Calculate the bucket from a timestamp.
@@ -158,7 +188,7 @@ class GregorianTime(object):
 
     return int(dt.strftime( self.FORMATS[self._step] ))
 
-  def from_bucket(self, bucket):
+  def from_bucket(self, bucket, native=False):
     '''
     Calculate the timestamp given a bucket.
     '''
@@ -171,6 +201,8 @@ class GregorianTime(object):
       normal = datetime(year=int(year), month=1, day=1) + timedelta(weeks=int(week))
     else:
       normal = datetime.strptime(bucket, self.FORMATS[self._step])
+    if native:
+      return normal
     return long(time.mktime( normal.timetuple() ))
 
   def buckets(self, start, end):
@@ -215,12 +247,18 @@ class GregorianTime(object):
       if relative_time:
         rtime = self.to_bucket(relative_time)
         ntime = self.to_bucket(time.time())
+
+        # Convert to number of days
+        day_diff = (self.from_bucket(ntime, native=True) - self.from_bucket(rtime, native=True)).days
+        # Convert steps to number of days as well
+        step_diff = (steps*SIMPLE_TIMES[self._step[0]]) / SIMPLE_TIMES['d']
+
         # The relative time is beyond our TTL cutoff
-        if (ntime - rtime) > steps:
+        if day_diff > step_diff:
           return 0
         # The relative time is in "recent" past or future
         else:
-          return (steps + rtime - ntime) * SIMPLE_TIMES[ self._step[0] ]
+          return (step_diff - day_diff) * SIMPLE_TIMES['d']
       return steps * SIMPLE_TIMES[ self._step[0] ]
 
     return None
@@ -556,13 +594,20 @@ class Timeseries(object):
     else:
       rval = self._get( name, interval, config, timestamp, fetch=fetch, process_row=process_row )
 
-    # If condensed, collapse the result into a single row
+    # If condensed, collapse the result into a single row. Adjust the step_size
+    # calculation to match.
+    if config['coarse']:
+      step_size = config['i_calc'].step_size(timestamp)
+    else:
+      step_size = config['r_calc'].step_size(timestamp)
     if condense and not config['coarse']:
       condense = condense if callable(condense) else self._condense
       rval = { config['i_calc'].normalize(timestamp) : condense(rval) }
+      step_size = config['i_calc'].step_size(timestamp)
+
     if transform:
       for k,v in rval.items():
-        rval[k] = self._process_transform(v, transform)
+        rval[k] = self._process_transform(v, transform, step_size)
     return rval
 
   def _get(self, name, interval, config, timestamp, fetch):
@@ -651,25 +696,25 @@ class Timeseries(object):
         for key in rval.iterkeys():
           data = condense( rval[key] )
           if transform and not collapse:
-            data = self._process_transform(data, transform)
+            data = self._process_transform(data, transform, config['i_calc'].step_size(key))
           rval[key] = data
       elif transform:
         for interval,resolutions in rval.items():
           for key in resolutions.iterkeys():
-            resolutions[key] = self._process_transform(resolutions[key], transform)
+            resolutions[key] = self._process_transform(resolutions[key], transform, config['r_calc'].step_size(key))
 
     if config['coarse'] or collapse:
       if collapse:
         collapse = collapse if callable(collapse) else condense if callable(condense) else self._condense
         data = collapse(rval)
         if transform:
-          rval = { rval.keys()[0] : self._process_transform(data, transform) }
+          rval = { rval.keys()[0] : self._process_transform(data, transform, config['i_calc'].step_size(rval.keys()[0], rval.keys()[-1])) }
         else:
           rval = { rval.keys()[0] : data }
 
       elif transform:
         for key,data in rval.items():
-          rval[key] = self._process_transform(data, transform)
+          rval[key] = self._process_transform(data, transform, config['i_calc'].step_size(key))
 
     return rval
 
@@ -699,17 +744,17 @@ class Timeseries(object):
           rval[i_key][r_key] = join( [res.get(i_key,{}).get(r_key) for res in results] )
     return rval
 
-  def _process_transform(self, data, transform):
+  def _process_transform(self, data, transform, step_size):
     '''
     Process transforms on the data.
     '''
     if isinstance(transform, (list,tuple,set)):
-      return { t : self._transform(data,t) for t in transform }
+      return { t : self._transform(data,t,step_size) for t in transform }
     elif isinstance(transform, dict):
-      return { tn : self._transform(data,tf) for tn,tf in transform.items() }
-    return self._transform(data, transform)
+      return { tn : self._transform(data,tf,step_size) for tn,tf in transform.items() }
+    return self._transform(data,transform,step_size)
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
@@ -752,7 +797,7 @@ class Series(Timeseries):
   def _type_no_value(self):
     return []
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
@@ -769,8 +814,10 @@ class Series(Timeseries):
       data = max( data or [0])
     elif transform=='sum':
       data = sum( data )
+    elif transform=='rate':
+      data = len( data ) / float(step_size)
     elif callable(transform):
-      data = transform(data)
+      data = transform(data, step_size)
     return data
 
   def _process_row(self, data):
@@ -805,7 +852,7 @@ class Histogram(Timeseries):
   def _type_no_value(self):
     return {}
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
@@ -822,8 +869,10 @@ class Histogram(Timeseries):
       data = max(data.keys() or [0])
     elif transform=='sum':
       data = sum( k*v for k,v in data.items() )
+    elif transform=='rate':
+      data = { k:v/float(step_size) for k,v in data.items() }
     elif callable(transform):
-      data = transform(data)
+      data = transform(data, step_size)
     return data
 
   def _process_row(self, data):
@@ -862,13 +911,15 @@ class Count(Timeseries):
   def _type_no_value(self):
     return 0
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
     '''
-    if callable(transform):
-      data = transform(data)
+    if transform=='rate':
+      data = data / float(step_size)
+    elif callable(transform):
+      data = transform(data, step_size)
     return data
 
   def insert(self, name, value=1, timestamp=None, **kwargs):
@@ -903,13 +954,13 @@ class Gauge(Timeseries):
     # TODO: resolve this disconnect with redis backend
     return 0
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
     '''
     if callable(transform):
-      data = transform(data)
+      data = transform(data, step_size)
     return data
 
   def _process_row(self, data):
@@ -944,7 +995,7 @@ class Set(Timeseries):
   def _type_no_value(self):
     return set()
 
-  def _transform(self, data, transform):
+  def _transform(self, data, transform, step_size):
     '''
     Transform the data. If the transform is not supported by this series,
     returns the data unaltered.
@@ -961,6 +1012,8 @@ class Set(Timeseries):
       data = max(data or [0])
     elif transform=='sum':
       data = sum(data)
+    elif transform=='rate':
+      data = len(data) / float(step_size)
     elif callable(transform):
       data = transform(data)
     return data
